@@ -26,8 +26,10 @@ import (
 	"github.com/snapcore/snapd/interfaces"
 	"github.com/snapcore/snapd/interfaces/apparmor"
 	"github.com/snapcore/snapd/interfaces/kmod"
+	"github.com/snapcore/snapd/interfaces/mount"
 	"github.com/snapcore/snapd/interfaces/seccomp"
 	"github.com/snapcore/snapd/interfaces/udev"
+	"github.com/snapcore/snapd/osutil"
 	"github.com/snapcore/snapd/snap"
 )
 
@@ -53,17 +55,20 @@ type commonInterface struct {
 	connectedPlugAppArmor  string
 	connectedPlugSecComp   string
 	connectedPlugUDev      []string
-	reservedForOS          bool
 	rejectAutoConnectPairs bool
 
+	connectedPlugUpdateNSAppArmor string
+	connectedPlugMount            []osutil.MountEntry
+
 	connectedPlugKModModules []string
 	connectedSlotKModModules []string
 	permanentPlugKModModules []string
 	permanentSlotKModModules []string
 
-	usesPtraceTrace     bool
-	suppressPtraceTrace bool
-	suppressHomeIx      bool
+	usesPtraceTrace      bool
+	suppressPtraceTrace  bool
+	suppressHomeIx       bool
+	controlsDeviceCgroup bool
 }
 
 // Name returns the interface name.
@@ -83,17 +88,6 @@ func (iface *commonInterface) StaticInfo() interfaces.StaticInfo {
 	}
 }
 
-// BeforePrepareSlot checks and possibly modifies a slot.
-//
-// If the reservedForOS flag is set then only slots on core snap
-// are allowed.
-func (iface *commonInterface) BeforePrepareSlot(slot *snap.SlotInfo) error {
-	if iface.reservedForOS {
-		return sanitizeSlotReservedForOS(iface, slot)
-	}
-	return nil
-}
-
 func (iface *commonInterface) AppArmorConnectedPlug(spec *apparmor.Specification, plug *interfaces.ConnectedPlug, slot *interfaces.ConnectedSlot) error {
 	if iface.usesPtraceTrace {
 		spec.SetUsesPtraceTrace()
@@ -103,8 +97,11 @@ func (iface *commonInterface) AppArmorConnectedPlug(spec *apparmor.Specification
 	if iface.suppressHomeIx {
 		spec.SetSuppressHomeIx()
 	}
-	if iface.connectedPlugAppArmor != "" {
-		spec.AddSnippet(iface.connectedPlugAppArmor)
+	if snippet := iface.connectedPlugAppArmor; snippet != "" {
+		spec.AddSnippet(snippet)
+	}
+	if snippet := iface.connectedPlugUpdateNSAppArmor; snippet != "" {
+		spec.AddUpdateNS(snippet)
 	}
 	return nil
 }
@@ -136,6 +133,15 @@ func (iface *commonInterface) KModConnectedSlot(spec *kmod.Specification, plug *
 	return nil
 }
 
+func (iface *commonInterface) MountConnectedPlug(spec *mount.Specification, plug *interfaces.ConnectedPlug, slot *interfaces.ConnectedSlot) error {
+	for _, entry := range iface.connectedPlugMount {
+		if err := spec.AddMountEntry(entry); err != nil {
+			return err
+		}
+	}
+	return nil
+}
+
 func (iface *commonInterface) KModPermanentPlug(spec *kmod.Specification, plug *snap.PlugInfo) error {
 	for _, m := range iface.permanentPlugKModModules {
 		if err := spec.AddModule(m); err != nil {
@@ -162,8 +168,14 @@ func (iface *commonInterface) SecCompConnectedPlug(spec *seccomp.Specification,
 }
 
 func (iface *commonInterface) UDevConnectedPlug(spec *udev.Specification, plug *interfaces.ConnectedPlug, slot *interfaces.ConnectedSlot) error {
-	for _, rule := range iface.connectedPlugUDev {
-		spec.TagDevice(rule)
+	// don't tag devices if the interface controls it's own device cgroup
+	if iface.controlsDeviceCgroup {
+		spec.SetControlsDeviceCgroup()
+	} else {
+		for _, rule := range iface.connectedPlugUDev {
+			spec.TagDevice(rule)
+		}
 	}
+
 	return nil
 }
