@@ -27,6 +27,7 @@ import (
 	"sync"
 
 	"github.com/snapcore/snapd/asserts"
+	"github.com/snapcore/snapd/gadget"
 	"github.com/snapcore/snapd/i18n"
 	"github.com/snapcore/snapd/logger"
 	"github.com/snapcore/snapd/netutil"
@@ -139,11 +140,11 @@ func canAutoRefresh(st *state.State) (bool, error) {
 	return true, nil
 }
 
-func checkGadgetOrKernel(st *state.State, snapInfo, curInfo *snap.Info, flags snapstate.Flags, deviceCtx snapstate.DeviceContext) error {
+func checkGadgetOrKernel(st *state.State, snapInfo, curInfo *snap.Info, _ snap.Container, flags snapstate.Flags, deviceCtx snapstate.DeviceContext) error {
 	kind := ""
 	var snapType snap.Type
 	var getName func(*asserts.Model) string
-	switch snapInfo.GetType() {
+	switch snapInfo.Type() {
 	case snap.TypeGadget:
 		kind = "gadget"
 		snapType = snap.TypeGadget
@@ -202,11 +203,29 @@ func checkGadgetOrKernel(st *state.State, snapInfo, curInfo *snap.Info, flags sn
 	return nil
 }
 
+func checkGadgetValid(st *state.State, snapInfo, _ *snap.Info, snapf snap.Container, flags snapstate.Flags, deviceCtx snapstate.DeviceContext) error {
+	if snapInfo.Type() != snap.TypeGadget {
+		// not a gadget, nothing to do
+		return nil
+	}
+	if deviceCtx.ForRemodeling() {
+		// in this case the gadget is checked by
+		// checkGadgetRemodelCompatible
+		return nil
+	}
+
+	// do basic validity checks on the gadget against its model constraints
+	_, err := gadget.ReadInfoFromSnapFile(snapf, deviceCtx.Model())
+	return err
+}
+
 var once sync.Once
 
 func delayedCrossMgrInit() {
 	once.Do(func() {
 		snapstate.AddCheckSnapCallback(checkGadgetOrKernel)
+		snapstate.AddCheckSnapCallback(checkGadgetValid)
+		snapstate.AddCheckSnapCallback(checkGadgetRemodelCompatible)
 	})
 	snapstate.CanAutoRefresh = canAutoRefresh
 	snapstate.CanManageRefreshes = CanManageRefreshes
@@ -318,11 +337,20 @@ func extractDownloadInstallEdgesFromTs(ts *state.TaskSet) (firstDl, lastDl, firs
 	return firstDl, tasks[edgeTaskIndex], tasks[edgeTaskIndex+1], lastInst, nil
 }
 
+func notInstalled(st *state.State, name string) (bool, error) {
+	_, err := snapstate.CurrentInfo(st, name)
+	_, isNotInstalled := err.(*snap.NotInstalledError)
+	if isNotInstalled {
+		return true, nil
+	}
+	return false, err
+}
+
 func remodelTasks(ctx context.Context, st *state.State, current, new *asserts.Model, deviceCtx snapstate.DeviceContext, fromChange string) ([]*state.TaskSet, error) {
 	userID := 0
 	var tss []*state.TaskSet
 
-	// adjust kernel track
+	// kernel
 	if current.Kernel() == new.Kernel() && current.KernelTrack() != new.KernelTrack() {
 		ts, err := snapstateUpdateWithDeviceContext(st, new.Kernel(), &snapstate.RevisionOptions{Channel: new.KernelTrack()}, userID, snapstate.Flags{NoReRefresh: true}, deviceCtx, fromChange)
 		if err != nil {
@@ -330,15 +358,48 @@ func remodelTasks(ctx context.Context, st *state.State, current, new *asserts.Mo
 		}
 		tss = append(tss, ts)
 	}
-	// add new kernel
+
+	var ts *state.TaskSet
 	if current.Kernel() != new.Kernel() {
-		// TODO: we need to support corner cases here like:
-		//  0. start with "old-kernel"
-		//  1. remodel to "new-kernel"
-		//  2. remodel back to "old-kernel"
-		// In step (2) we will get a "already-installed" error
-		// here right now (workaround: remove "old-kernel")
-		ts, err := snapstateInstallWithDeviceContext(ctx, st, new.Kernel(), &snapstate.RevisionOptions{Channel: new.KernelTrack()}, userID, snapstate.Flags{}, deviceCtx, fromChange)
+		needsInstall, err := notInstalled(st, new.Kernel())
+		if err != nil {
+			return nil, err
+		}
+		if needsInstall {
+			ts, err = snapstateInstallWithDeviceContext(ctx, st, new.Kernel(), &snapstate.RevisionOptions{Channel: new.KernelTrack()}, userID, snapstate.Flags{}, deviceCtx, fromChange)
+		} else {
+			ts, err = snapstate.LinkNewBaseOrKernel(st, new.Base())
+		}
+		if err != nil {
+			return nil, err
+		}
+		tss = append(tss, ts)
+	}
+	if current.Base() != new.Base() {
+		needsInstall, err := notInstalled(st, new.Base())
+		if err != nil {
+			return nil, err
+		}
+		if needsInstall {
+			ts, err = snapstateInstallWithDeviceContext(ctx, st, new.Base(), nil, userID, snapstate.Flags{}, deviceCtx, fromChange)
+		} else {
+			ts, err = snapstate.LinkNewBaseOrKernel(st, new.Base())
+		}
+		if err != nil {
+			return nil, err
+		}
+		tss = append(tss, ts)
+	}
+	// gadget
+	if current.Gadget() == new.Gadget() && current.GadgetTrack() != new.GadgetTrack() {
+		ts, err := snapstateUpdateWithDeviceContext(st, new.Gadget(), &snapstate.RevisionOptions{Channel: new.GadgetTrack()}, userID, snapstate.Flags{NoReRefresh: true}, deviceCtx, fromChange)
+		if err != nil {
+			return nil, err
+		}
+		tss = append(tss, ts)
+	}
+	if current.Gadget() != new.Gadget() {
+		ts, err := snapstateInstallWithDeviceContext(ctx, st, new.Gadget(), &snapstate.RevisionOptions{Channel: new.GadgetTrack()}, userID, snapstate.Flags{}, deviceCtx, fromChange)
 		if err != nil {
 			return nil, err
 		}
@@ -350,16 +411,17 @@ func remodelTasks(ctx context.Context, st *state.State, current, new *asserts.Mo
 	for _, snapRef := range new.RequiredNoEssentialSnaps() {
 		// TODO|XXX: have methods that take refs directly
 		// to respect the snap ids
-		_, err := snapstate.CurrentInfo(st, snapRef.SnapName())
-		// If the snap is not installed we need to install it now.
-		if _, ok := err.(*snap.NotInstalledError); ok {
+		needsInstall, err := notInstalled(st, snapRef.SnapName())
+		if err != nil {
+			return nil, err
+		}
+		if needsInstall {
+			// If the snap is not installed we need to install it now.
 			ts, err := snapstateInstallWithDeviceContext(ctx, st, snapRef.SnapName(), nil, userID, snapstate.Flags{Required: true}, deviceCtx, fromChange)
 			if err != nil {
 				return nil, err
 			}
 			tss = append(tss, ts)
-		} else if err != nil {
-			return nil, err
 		}
 	}
 	// TODO: Validate that all bases and default-providers are part
@@ -438,9 +500,10 @@ func remodelTasks(ctx context.Context, st *state.State, current, new *asserts.Mo
 //
 // TODO:
 // - Check estimated disk size delta
-// - Reapply gadget connections as needed
 // - Check all relevant snaps exist in new store
 //   (need to check that even unchanged snaps are accessible)
+// - Make sure this works with Core 20 as well, in the Core 20 case
+//   we must enforce the default-channels from the model as well
 func Remodel(st *state.State, new *asserts.Model) (*state.Change, error) {
 	var seeded bool
 	err := st.Get("seeded", &seeded)
@@ -459,6 +522,15 @@ func Remodel(st *state.State, new *asserts.Model) (*state.Change, error) {
 		return nil, fmt.Errorf("cannot remodel to different series yet")
 	}
 
+	// TODO:UC20: support remodel, also ensure we never remodel to a lower
+	// grade
+	if current.Grade() != asserts.ModelGradeUnset {
+		return nil, fmt.Errorf("cannot remodel Ubuntu Core 20 models yet")
+	}
+	if new.Grade() != asserts.ModelGradeUnset {
+		return nil, fmt.Errorf("cannot remodel to Ubuntu Core 20 models yet")
+	}
+
 	// TODO: we need dedicated assertion language to permit for
 	// model transitions before we allow cross vault
 	// transitions.
@@ -473,12 +545,9 @@ func Remodel(st *state.State, new *asserts.Model) (*state.Change, error) {
 	}
 
 	// calculate snap differences between the two models
-	// FIXME: this needs work to switch the base to boot as well
-	if current.Base() != new.Base() {
-		return nil, fmt.Errorf("cannot remodel to different bases yet")
-	}
-	if current.Gadget() != new.Gadget() {
-		return nil, fmt.Errorf("cannot remodel to different gadgets yet")
+	// FIXME: this needs work to switch from core->bases
+	if current.Base() == "" && new.Base() != "" {
+		return nil, fmt.Errorf("cannot remodel from core to bases yet")
 	}
 
 	// TODO: should we run a remodel only while no other change is
